(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     49844,       1308]*)
(*NotebookOutlinePosition[     50718,       1342]*)
(*  CellTagsIndexPosition[     50650,       1336]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[BoxData[
    RowBox[{"(*", "\n", 
      RowBox[{
        RowBox[{
          RowBox[{"Distributed", " ", 
            StyleBox["Mathematica",
              FontSlant->"Italic"], "\n", 
            StyleBox["Mathematica",
              FontSlant->"Italic"], " ", "interface", " ", "to", " ", "Java", 
            " ", "scheduler", "\n", "\n", \((c)\), " ", "2000", " ", "Cleo", 
            " ", "Pau", " ", \((RISC - Linz)\)}], ",", " ", 
          \(see\ file\ COPYRIGHT\)}], "\n", 
        \(See\ \(http : \) // 
          www . risc . uni - \(linz . ac . at/software\)/distmath\)}], "\n", 
      "*)"}]], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Off[General::"\<spell1\>"]; \)\)], "Input"],

Cell[BoxData[
    \(BeginPackage["\<dist`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(versionNumber::usage = "\<\>"; \nversion::usage = "\<\>"; \n
    isClient::usage = "\<\>"; \nkeys::usage = "\<\>"; \n
    fields::usage = "\<\>"; \nserverDepth::usage = "\<\>"; \n
    taskMin::usage = "\<\>"; \ntaskMax::usage = "\<\>"; \n
    taskId::usage = "\<\>"; \ninitialized::usage = "\<\>"; \n
    parameters::usage = 
      "\<parameters is a list {{name1,...},{value1,...}}, containing the \
names and the corresponding values of the global variables. The first global \
variable is \"proc\", and contains the number of active processors, including \
the local one.\>"; \n
    InitializeD::usage = 
      "\<InitializeD[{{machine1,type1},{machine2,type2},...}] initializes a \
distributed Mathematica session by starting remote processes, and \
establishing the corresponding network connections, where machine is the \
domain name (or numeric Internet address) of a machine, and type is the \
corresponding key in the file dist.systems, that describes how to contact \
this machine.\>"; \n
    TerminateD::usage = 
      "\<TerminateD[] terminates the current distributed Mathematica session \
by destroying all remote Mathematica kernels and closing all network \
connections.\>"; \nConnectMachineD::usage = "\<\>"; \n
    ConnectD::usage = "\<\>"; \nCheckVersionD::usage = "\<\>"; \n
    ErrorD::usage = "\<\>"; \nReadlineD::usage = "\<\>"; \n
    SendD::usage = "\<\>"; \nSendListD::usage = "\<\>"; \n
    ReceiveD::usage = "\<\>"\), 
    \(GetParamD::usage = 
      "\<GetParamD[\"name\"] gets the value of the global variable \"name\". \
If the global variable is not defined, then the session will end with an \
error.\>"; \n
    SetParamD::usage = 
      "\<SetParamD[\"name\"] sets the value of the global variable \"name\". \
If the global variable is not defined, then it is created, otherwise its \
value is reset with \"value\".\>"; \n
    HasParamD::usage = 
      "\<HasParamD[\"name\"] returns True if the global variable \"name\" is \
defined, and False otherwise.\>"; \n
    AllD::usage = 
      "\<AllD[\"command\"] executes command on every Mathematica kernel \
connected to the distributed session (including that one connected to the \
user interface).\>"; \n
    StartD::usage = 
      "\<StartD[\"command\"] creates a task that computes command on some \
Mathematica kernel connected to the distributed session.\nStartD[fun,args...] \
creates a task that computes fun[args...].\n Both return the task id\>"; \n
    ProcessD::usage = "\<ProcessD[]\>"; \n
    WaitD::usage = 
      "\<WaitD[task] suspends the current execution until task has completed \
and then returns its result.\>"; \n
    SelectD::usage = 
      "\<SelectD[tasklist] supsends the current execution until the execution \
of some task in taskList has completed; it then returns a selector for this \
task, i.e., a list {index, result}, where index denotes the position of the \
task in the list and result is its result.\>"; \n
    DeleteD::usage = 
      "\<Delete[task] deletes task from the distributed Mathematica \
session.\>"; \n
    FlushD::usage = 
      "\<Flush[code] flushes the internal result caches of the system. If \
code is 0, result caching is subsequently switched off. If code is 1, result \
caching remains switched on.\>"; \n
    ResetD::usage = 
      "\<ResetD[] resets the session, i.e., it deletes all tasks and their \
results, and flushes all caches; it also destroys and restarts all \
Mathematica kernels connected to the system, except for the kernel connected \
to the user interface.\>"; \n
    AbortD::usage = 
      "\<AbortD[message] aborts the session by terminating all Mathematica \
kernels, except for the kernel connected to the user interface, and closing \
all network connections. The denoted message is printed at the user \
interface.\>"; \nTraceD::usage = "\<\>"; \nLabelD::usage = "\<\>"; \n
    VisualizeD::usage = "\<\>"; \n
    DataD::usage = 
      "\<DataD[] creates a shared object that may be used to communicate data \
between different tasks, and returns its id.\>"; \n
    PutD::usage = 
      "\<PutD[d,v] puts the value v into the shared data object d and \
releases all tasks blocked on an attempt to get a value from this object.\>"; 
    \nGetD::usage = 
      "\<GetD[d] returns the value of the shared data object d; if d is still \
empty, the current task is blocked until a value is put into d.\>"; \n
    ClearD::usage = 
      "\<Clear[d] resets the shared data object d into the empty state.\>"; \n
    ClientD::usage = "\<\>"; \nServerD::usage = "\<\>"; \)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \( (*\ 
      in\ loc\ de\ o\ noua\ 
        \(functie : \n\t\t\t\(-\ msgValue\)\ folosesc\ ToExpresion\); \n\t\ \ 
      \(-\ expMessage\)\ folosesc\ ToString; \n\t\ \ 
      \(-\ intMessage\)\ folosesc\ ToString; \n\t\t*) \)], "Input"],

Cell[BoxData[
    \(Begin["\<`Private`\>"]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["versionNumber", "Input"],
  " contains the version number that appears in the copyright message. Should \
be change at every change of version."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(\(versionNumber = "\<1.0.0\>"; \)\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["version", "Input"],
  " contains the protocol identifier."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(\(dist`version = "\<dist.Scheduler.externalVersion 1.0.6\>"; \)\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  "If ",
  StyleBox["isClient", "Input"],
  " is set to ",
  StyleBox["True", "Input"],
  ", client loop is activated (see file end)."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(\(If[dist`isClient\  != True, \n\t
      Print["\<Distributed Mathematica V\>", versionNumber, 
        "\< (c) 2000 Cleopatra Pau (RISC-Linz)\>"]; \n\t
      \(Print["\<See http://www.risc.uni-linz.ac.at/software/distmath\>"];\), 
      \n\t\(dist`isClient = True;\), \n\t
      Print["\<Distributed Mathematica V\>", versionNumber, 
        "\< (c) 2000 Cleopatra Pau (RISC-Linz)\>"]; \n\t
      Print["\<See http://www.risc.uni-linz.ac.at/software/distmath\>"]; \n\t
      dist`isClient = False; \n]; \)\)], "Input",
  InitializationCell->True,
  CellTags->"1"],

Cell["\<\
The table of keywords, sent and received in the communication with \
the Java scheduler. For each key, the number of the fields required is \
written as a comment.\
\>", "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(\(dist`keys = {\n\t\t"\<okay\>",  (*\ 0\ *) \n\t\t
        "\<error\>",  (*\ 1\ *) \n\t\t"\<start\>",  (*\ 2\ *) \n\t\t
        "\<machine\>",  (*\ 4\ *) \n\t\t"\<task\>",  (*\ 2\ *) \n\t\t
        "\<quit\>",  (*\ 1\ *) \n\t\t"\<taskid\>",  (*\ 2\ *) \n\t\t
        "\<result\>",  (*\ 2\ *) \n\t\t"\<request\>",  (*\ 2\ *) \n\t\t
        "\<wait\>",  (*\ 1\ *) \n\t\t"\<reply\>",  (*\ 3\ *) \n\t\t
        "\<flush\>",  (*\ 0\ *) \n\t\t"\<reset\>",  (*\ 0\ *) \n\t\t
        "\<delete\>",  (*\ 1\ *) \n\t\t"\<all\>",  (*\ 1\ *) \n\t\t
        "\<abort\>",  (*\ 1\ *) \n\t\t"\<load\>",  (*\ 2\ *) \n\t\t
        "\<nwait\>",  (*\ \(-1\)\ *) \n\t\t"\<label\>",  (*\ 1\ *) \n\t\t
        "\<visual\>",  (*\ 4\ *) \n\t\t"\<data\>",  (*\ 1\ *) \n\t\t
        "\<put\>",  (*\ 2\ *) \n\t\t"\<clear\>",  (*\ 1\ *) \n\t\t
        "\<proc\>" (*\ 2\ *) \n\t}; \)\)], "Input",
  InitializationCell->True],

Cell["\<\
Field table listing the number of fields for each key (see \
above):\
\>", "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(\(dist`fields = \n
      \t{0, 1, 2, 4, 2, 1, 2, 2, 2, 1, 3, 0, 0, 1, 1, 1, 2, \(-1\), 1, 4, 1, 
        2, 1, 2}; \)\)], "Input",
  InitializationCell->True],

Cell["The depth of nestings of server calls.", "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(\(dist`serverDepth = 0; \)\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  "The range of task ids. ",
  StyleBox["taskMin", "Input"],
  " is minimum value, ",
  StyleBox["taskMax", "Input"],
  " is the maximum value, ",
  StyleBox["taskId", "Input"],
  " is the current value."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`taskMin = 0; \ndist`taskMax = 0; \ndist`taskId = 0; \)], "Input",
  InitializationCell->True],

Cell["\<\
The initialization status, that shows if there is a connection with \
remote kernels or not.\
\>", "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(\(dist`initialized = False; \)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(dist`parameters = {{}, {}}; \)\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["InitializeD[{{name, type},...}]", "Input"],
  " initialize the system and starts the connections with the others \
machines. If the system is already initialized, then this command closes the \
previous connection and starts a new one.\nThe connection is fixed as \
follows:\n- for the output is created a pipe with the Unix command ",
  StyleBox["mkfifo", "Input"],
  ". The name of this pipe is ",
  StyleBox["/tmp/distmath.out", "Input"],
  StyleBox["number",
    FontSlant->"Italic"],
  ", where ",
  StyleBox["number",
    FontSlant->"Italic"],
  " is uniquely created in the variable ",
  StyleBox["name", "Input"],
  ". The pipe is opened afterwards, with the corresponding output stream ",
  StyleBox["cout", "Input"],
  ".\n- for the input, ",
  StyleBox["OpenRead", "Input"],
  " runs the external ",
  StyleBox["command ", "TI"],
  StyleBox["java", "Input"],
  ", and opens a pipe to get input from it, with the corresponding output \
stream ",
  StyleBox["cin", "Input"],
  ".\nIf ",
  StyleBox["tracefile", "Input"],
  " has a value, than java is started using it for tracing the parallel \
execution of the program.\nAfter the two pipes are opened, the version is \
compared with that one sent by the scheduler. Then it waits for the message ",
  
  StyleBox["taskid", "Input"],
  " from the scheduler, with the corresponding values, and it starts the \
connection with the machines. ",
  StyleBox["dist`initialized", "Input"],
  " is set on ",
  StyleBox["True", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(SetAttributes[dist`InitializeD, HoldAllComplete]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dist`InitializeD[machines_] := 
      Module[\n\t{reply, name}, \n\t
        If[dist`initialized, \n\t\t\t
          Print["\<previous connection is terminated\>"]; \n\t\t\t
          dist`TerminateD[]; \n\t]; \n\t\tname = << "\<!sh -c 'echo $$'\>"; 
        \t\tdist`outname = 
          StringJoin["\</tmp/distmath.out\>", ToString[name]]; \n\t\t
        Run["\<mkfifo \>"<>dist`outname]; \n\t\t
        If[Length[OwnValues[dist`tracefile]] != 0, \n\t\t\t
          \(dist`cin = 
            OpenRead[
              "\<!java dist.Scheduler local - \>"<>dist`outname<>"\< \>"<>
                dist`tracefile]; \), \n\t\t\t
          \(dist`cin = 
            OpenRead["\<!java dist.Scheduler local - \>"<>dist`outname]; \)\n
          \t\t]; \n\t\tdist`cout = OpenWrite[dist`outname]; \n\t\t\n
        \t\t (*\ If\ not\ set, \ PageWidth\ has\ a\ finite\ value, \ 
          and\ it\ will\ break\ the\ lines\ sent\ to\ the\ java\ scheduler, \ 
          causing\ lots\ of\ errors\ *) \n\t\t\n\t\t
        SetOptions[dist`cout, PageWidth -> Infinity]; \n\t\t
        dist`CheckVersionD[]; \n\t\treply = dist`ReceiveD[]; \n\t\t
        If[reply[\([1]\)] != "\<taskid\>", \n\t\t\t
          \(dist`ErrorD["\<Unexpected message (\>"<>reply[\([1]\)]<>"\<)\>"]; 
          \)\n\t\t\t]; \n\t\tdist`TaskidMessage[reply]; \n\t\t
        dist`ConnectD[machines]; \n\t\tdist`initialized = True; \n\t\t
        AppendTo[dist`parameters[\([1]\)], "\<proc\>"]; \n\t\t
        AppendTo[dist`parameters[\([2]\)], Length[machines] + 1]; \n\t\t
        "\<okay\>"\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["TerminateD[]", "Input"],
  " terminates the system, closing the pipes and the connections."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`TerminateD[] := 
      Module[\n\t\t{reply}, \n\t\t
        If[dist`initialized != True, \n\t\t\t
          \(dist`ErrorD["\<The system is not connected\>"]; \), \n\t\t\t
          dist`SendD["\<quit\>", "\<0\>"]; \n\t\t\treply = dist`ReceiveD[]; \n
          \t\t\tIf[reply[\([1]\)] != "\<quit\>", \n\t\t\t\t
            \(dist`ErrorD[
              "\<Unexpected message (\>"<>reply[\([1]\)]<>"\<)\>"]; \)\n
            \t\t\t\t]; \n\t\t\tClose[dist`cin]; \n\t\t\tClose[dist`cout]; \n
          \t\t\tRun["\<rm \>"<>dist`outname]; \n\t\t\t
          \(dist`initialized = False;\), \n\t\t\t
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        "\<okay\>"\n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ConnectMachineD[name,type]", "Input"],
  " makes the connection with the machine ",
  StyleBox["name", "Input"],
  ". It sends the messages ",
  StyleBox["start", "Input",
    FontFamily->"Terminal"],
  ", followed by ",
  StyleBox["name", "Input"],
  " and ",
  StyleBox["type", "Input"],
  ". Depending on the type, the scheduler starts on the specified machine \
another java scheduler and a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " kernel. After sending the message, it waits the answer, that should be ",
  StyleBox["okay", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(SetAttributes[dist`ConnectMachineD, HoldAllComplete]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dist`ConnectMachineD[machine_, id_] := 
      Module[\n\t\t{reply}, \n\t\t
        dist`SendD["\<start\>", ToString[machine], ToString[id]]; \n\t\t
        reply = dist`ReceiveD[]; \n\t\tkey = reply[\([1]\)]; \n\t\t
        If[key == "\<error\>", \n\t\t\t
          Print["\<        failed: \>"<>reply[\([2]\)]], \n\t\t\t
          If[key != "\<okay\>", \n\t\t\t\t
            \(dist`ErrorD[
              "\<Unexpected message (\>"<>ToString[key]<>"\<)\>"]; \)\n
            \t\t\t\t]\n\t\t\t]; \n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ConnectD[{{name,type},...}]", "Input"],
  " calls ",
  StyleBox["ConnectMachineD", "Input"],
  " for each pair ",
  StyleBox["{name,type}", "Input"],
  " of the parameters list."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(SetAttributes[dist`ConnectD, HoldAllComplete]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dist`ConnectD[machines_] := 
      Module[\n\t\t{i, machine, name, key}, \n\t\t
        For[i = 1, i <= Length[machines], \(i++\), \n\t\t\t
          machine = machines[\([i]\)]; \n\t\t\t
          name = StringReplace[ToString[machines[\([i, 1]\)]], 
              "\< \>" -> "\<\>"]; \n\t\t\tkey = machine[\([2]\)]; \n\t\t\t
          Print["\<Connecting \>"<>ToString[name]<>"\<...\>"]; \n\t\t\t
          dist`ConnectMachineD[name, key]; \n\t\t\t]; \n\t\t"\<okay\>"\n\t\t]
        \)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["CheckVersionD[]", "Input"],
  " sends ",
  StyleBox["version", "Input"],
  " to the scheduler, waits for the answer, and then it compares them. They \
should be identical."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`CheckVersionD[] := 
      Module[\n\t\t{vers}, \n\t\tWrite[dist`cout, OutputForm[dist`version]]; 
        \n\t\tvers = dist`ReadlineD[]; \n\t\t
        If[vers != dist`version, \n\t\t\t
          \(dist`ErrorD[
            "\<Invalid protocol ('\>"<>dist`version<>"\<' expected but '\>"<>
              vers<>"\<' received)\>"]; \)\n\t\t\t]; \n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ErrorD[message]", "Input"],
  " should print ",
  StyleBox["message", "Input"],
  " and then close the pipes and the connections. It does not work properly."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ErrorD[message_] := 
      Module[\n\t\t{}, \n\t\tIf[dist`isClient, \n\t\t\tQuit[]\n\t\t\t]; \n\t\t
        Print[message]; \n\t\t
        If[dist`initialized == True, \n\t\t\t
          If[dist`cout != $Output, \n\t\t\t\t\(Close[dist`cout]; \)\n\t\t\t]; 
          \n\t\t\tIf[ValueQ[dist`cin], \n\t\t\t\t\(Close[dist`cin]; \)\n
            \t\t\t]; \n\t\t\tRun["\<rm \>"<>dist`outname]; \n\t\t\t
          dist`initialized = False; \n\t\t\tAbort[]; \n\t\t\t]\n\t\t]\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ReadlineD[]", "Input"],
  " reads a line from the input pipe, and returns to the point where it was \
called."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ReadlineD[] := 
      Module[\n\t\t{reply}, \n\t\t
        If[dist`isClient, \n\t\t\t\n
          \t\t\t (*\ In\ the\ client\ case, \ 
            the\ input\ is\ read\ from\ the\ standard\ input, \ 
            as\ a\ string . \ The\ argument\ represents\ the\ prompter, \ 
            which\ is\ the\ empty\ string\ \(here . \)\ *) \n\t\t\t\n\t\t\t
          \(reply = InputString["\<\>"]; \), \n\t\t\t
          \(reply = Read[dist`cin, String]; \)\n\t\t\t]; \n\t\treply\n\t\t]
        \)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["SendD[arg1, ...]", "Input"],
  " constructs a string from arguments separated by ",
  StyleBox["newline",
    FontSlant->"Italic"],
  ", and sends the string to the scheduler. It is used to send messages. Each \
arguments should be a string."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`SendD[arguments___] := 
      Module[\n\t\t{i, message}, \n\t\tmessage = {arguments}[\([1]\)]; \n\t\t
        For[i = 2, i <= Length[{arguments}], \(i++\), \n\t\t\t
          \(message = message<>"\<\n\>"<>{arguments}[\([i]\)]; \)\n\t\t\t]; \n
        \t\t (*Write[$Messages, message]; \n\t\t
          Write[$Messages, "\<terminat\>"]; *) \n\t\t
        Write[dist`cout, OutputForm[message]]; \n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["SendListD[key,list]", "Input"],
  " is similar to ",
  StyleBox["SendD", "Input"],
  ", but the arguments are a string and a list of integers. It is used only \
to send the message ",
  StyleBox["nwait", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`SendListD[key_, list_] := 
      Module[\n\t\t{len, message, i}, \n\t\tlen = Length[list]; \n\t\t
        message = key<>"\<\n\>"<>ToString[len]; \n\t\t
        For[i = 1, i <= len, \(i++\), \n\t\t\t
          \(message = message<>"\<\n\>"<>ToString[list[\([i]\)]]; \)\n
          \t\t\t]; \n\t\tWrite[dist`cout, OutputForm[message]]; \n\t\t]\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ReceiveD[]", "Input"],
  " receives a message from the scheduler and construct a list from it. The \
first line should contain a keyword that exists in the table ",
  StyleBox["keys", "Input"],
  ". After it identifies the key, it read a number of lines, corresponding to \
the number of fields of the key. All the keys except ",
  StyleBox["nwait", "Input"],
  " have positive number of fields. The entrance corresponding to ",
  StyleBox["nwait", "Input"],
  " is -1, and in this case the number of the fields is given on the second \
line which is read."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ReceiveD[] := 
      Module[\n\t\t{key, list, number, field, i}, \n\t\t
        key = dist`ReadlineD[]; \n\t\t
        If[FreeQ[dist`keys, key], \n\t\t\t
          \(dist`ErrorD["\<Invalid message key (\>"<>ToString[key]<>"\<)\>"]; 
          \)\n\t\t\t]; \n\t\tlist = {key}; \n\t\t
        number = dist`fields[\([\(Position[dist`keys, key]\)[\([1, 1]\)]]\)]; 
        \n\t\tIf[number == \(-1\), \n\t\t\tfield = dist`ReadlineD[]; \n\t\t\t
          number = dist`MsgInteger[field]; \ \n\t\t]; \n\t\t
        For[i = 1, i <= number, \(i++\), \n\t\t\tfield = dist`ReadlineD[]; \n
          \t\t\t (*Print["\<mesaj[\>"<>ToString[i]<>"\<] e: \>"<>field]; *) \n
          \t\t\tlist = Append[list, field]; \n\t\t\t]; \n\t\tlist\n\t\t]\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["MsgInteger[message_]", "Input"],
  " is only a sinonym for ToExpression. It could be removed, but not before \
the replacement of all the appearances."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`MsgInteger[message_] := 
      Module[\n\t\t{}, \n\t\tToExpression[message]\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ExpMessage[name,arg_list]", "Input"],
  " constructs the expression that will be send for evaluation as a task. To \
avoid the evaluation, first is set a ",
  StyleBox["Hold", "Input"],
  ", then the arguments are replaced by ",
  StyleBox["arg_list", "Input"],
  ", and the name of the function is replaced by ",
  StyleBox["name", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ExpMessage[fName_, fArg_] := 
      Module[\n\t\t{expr}, \n\t\texpr = Hold[functionName[args]]; \n\t\t
        expr = ReplacePart[expr, fArg, {{1, 1}}]; \n\t\t
        ToString[ReplacePart[expr, fName, {{1, 0}}], InputForm]\n\t]\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["GetParamD[name]", "Input"],
  " gets the value of the global variable ",
  StyleBox["name",
    FontWeight->"Bold"],
  ". If the global variable is not defined then the session ends with an \
error. ",
  StyleBox["name",
    FontWeight->"Bold"],
  " should be a string."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`GetParamD[key_String] := 
      Module[\n\t\t{p}, \n\t\t
        If[\(! MemberQ[dist`parameters[\([1]\)], key]\), \n\t\t\t
          \(dist`ErrorD["\<The requested parameter is not set\>"]; \)\n\t\t]; 
        \n\t\tp = \(Position[dist`parameters[\([1]\)], key]\)[\([1, 1]\)]; \n
        \t\tdist`parameters[\([2, p]\)]\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["SetParamD[name,value]", "Input"],
  " sets the value of the global variable ",
  StyleBox["name",
    FontWeight->"Bold"],
  ". If the global variable is not defined then it is created, otherwise its \
value is reset with ",
  StyleBox["value",
    FontWeight->"Bold"],
  ". ",
  StyleBox["name",
    FontWeight->"Bold"],
  " should be a string."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`SetParamD[key_String, value_] := 
      Module[\n\t\t{p}, \n\t\t
        \(If[key == "\<proc\>", \n\t\t\t
          \(dist`ErrorD["\<Cannot set parameter proc (protected)\>"];\), \n
          \t\t\t\(If[MemberQ[dist`parameters[\([1]\)], key], \n\t\t\t\t
              p = \(Position[dist`parameters[\([1]\)], key]\)[\([1, 1]\)]; \n
              \t\t\t\t\(dist`parameters[\([2, p]\)] = value;\), \n\t\t\t\t
              AllD["\<AppendTo[dist`parameters[[1]],\"\>"<>key<>"\<\"]\>"]; \n
              \t\t\t\t\(AllD[
                  "\<AppendTo[dist`parameters[[2]],\>"<>ToString[value]<>
                    "\<]\>"];\), \n\t\t\t\t
              \(Print["\<This is a bug, please report it: SetParamD - 1\>"]; 
              \)\n\t\t\t];\), \n\t\t\t
          \(Print["\<This is a bug, please report it: SetParamD - 2\>"]; \)\n
          \t\t]; \)\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["HasParamD[name]", "Input"],
  " returns ",
  StyleBox["True",
    FontWeight->"Bold"],
  " if the global variable ",
  StyleBox["name",
    FontWeight->"Bold"],
  " is defined, and ",
  StyleBox["False",
    FontWeight->"Bold"],
  " otherwise. ",
  StyleBox["name",
    FontWeight->"Bold"],
  " should be a string."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`HasParamD[key_String] := 
      Module[\n\t\t{returnValue}, \n\t\t
        If[MemberQ[dist`parameters[\([1]\)], key], \n\t\t\t
          \(returnValue = True;\), \n\t\t\t\(returnValue = False;\), \n\t\t\t
          \(Print["\<This is a bug, please report it\>"]; \)\n\t\t]; \n\t\t
        returnValue\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["AllD[\"command\"]", "Input"],
  " sends ",
  StyleBox["command", "Input"],
  " to all the processes and executes it. It is useful for having global \
variables, or for loading files on all the kernels."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`AllD[command_] := 
      Module[\n\t\t{reply, key}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<all\>", command]; \n\t\treply = dist`ReceiveD[]; \n\t\t
        key = reply[\([1]\)]; \n\t\t
        If[key == "\<all\>", \n\t\t\t\(dist`AllMessage[reply]; \), \n\t\t\t
          \(dist`ErrorD["\<Unexpected message (\>"<>key<>"\<)\>"]; \)\n\t\t]; 
        \n\t\t"\<okay\>"\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["StartD[\"command\"]", "Input"],
  " creates a task, which is sent to the scheduler. The scheduler sends it to \
one of the free kernels to be executed. It returnes the number of the task. \
The command is first processed as follows:\n- a ",
  StyleBox["Hold", "Input"],
  " is put on the expression when the string is eliminated.\n- the head of \
the function is separated from the arguments\n- the arguments are put in a \
list and the Hold is released, so that they are evaluated\n- the new form of \
the command is constructed from the head and the evaluated arguments put in a \
list, and it is sent as a task to the scheduler.\nThe task consists in the \
evaluation of the outermost function of the command. All the other \
evaluations will be done on the local kernel.\n",
  StyleBox["Warning:",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  " If the command contains local variables of the function that starts the \
call, this function should be a ",
  StyleBox["Block", "Input"],
  " and not a ",
  StyleBox["Module", "Input"],
  ", to allow the evaluation of the local variables. If the commands contains \
arguments of the function that starts the call, their value cannot be known \
by the task and they remain unevaluated."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    StyleBox[
      \(dist`StartD[expr_String]\  := 
        Module[\n\t\t{e, \ fName, \ fArg, \ t, message}, \n\t\t
          If[dist`initialize\  != \ True, \ 
            \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n
          \t\te\  = \ ToExpression[expr, \ InputForm, \ Hold]; \n\t\t
          fName\  = \ e[\([1, 0]\)]; \n\t\t
          fArg\  = \ ReleaseHold[ReplacePart[e, \ List, \ {{1, \ 0}}]]; \n\t\t
          If[dist`taskId\  == \ dist`taskMax, \ 
            \(dist`ErrorD["\<Task identifier range overflow\>"]; \)\n\t\t]; \n
          \t\tt\  = \ dist`taskId; \n\t\t
          message\  = \ dist`ExpMessage[fName, \ fArg]; \n\t\t
          dist`SendD["\<task\>", \ ToString[t], \ message]; \n\t\t
          dist`taskId\  = \ t\  + \ 1; \n\t\tt\n\t]\),
      ShowStringCharacters->True,
      NumberMarks->True]], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["StartD[name,arg]", "Input"],
  " constructs the command that will be sent as a task. This is the form of ",
  
  StyleBox["Start", "Input"],
  " in ",
  StyleBox["Maple",
    FontSlant->"Italic"],
  ". It is useful because the user does not have to take care anymore about \
using ",
  StyleBox["Block", "Input"],
  " or ",
  StyleBox["Module", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`StartD[fName_, fArg___] := 
      Module[\n\t\t{t}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        If[dist`taskId\  == \ dist`taskMax, \ 
          \(dist`ErrorD["\<Task identifier range overflow\>"]; \)\n\t\t]; \n
        \t\tt\  = \ dist`taskId; \n\t\t
        message\  = \ dist`ExpMessage[fName, \ {fArg}]; \n\t\t
        dist`SendD["\<task\>", \ ToString[t], \ message]; \n\t\t
        dist`taskId\  = \ t\  + \ 1; \n\t\tt\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ProcessD[expr]", "Input"],
  " not yet documented."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ProcessD[expr_String] := 
      Module[\n\t\t{e, \ fName, \ fArg, \ t, message}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        e\  = \ ToExpression[expr, \ InputForm, \ Hold]; \n\t\t
        fName\  = \ e[\([1, 0]\)]; \n\t\t
        fArg\  = \ ReleaseHold[ReplacePart[e, \ List, \ {{1, \ 0}}]]; \n\t\t
        If[dist`taskId\  == \ dist`taskMax, \ 
          \(dist`ErrorD["\<Task identifier range overflow\>"]; \)\n\t\t]; \n
        \t\tt\  = \ dist`taskId; \n\t\t
        message\  = \ dist`ExpMessage[fName, \ fArg]; \n\t\t
        dist`SendD["\<proc\>", \ ToString[t], \ message]; \n\t\t
        dist`taskId\  = \ t\  + \ 1; \n\t\tt\t\t\n\t\t]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dist`ProcessD[fName_, fArg___] := 
      Module[\n\t\t{t}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        If[dist`taskId\  == \ dist`taskMax, \ 
          \(dist`ErrorD["\<Task identifier range overflow\>"]; \)\n\t\t]; \n
        \t\tt\  = \ dist`taskId; \n\t\t
        message\  = \ dist`ExpMessage[fName, \ {fArg}]; \n\t\t
        dist`SendD["\<proc\>", \ ToString[t], \ message]; \n\t\t
        dist`taskId\  = \ t\  + \ 1; \n\t\tt\t\t\n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["WaitD[task]", "Input"],
  " waits for a result. It first sends the message ",
  StyleBox["wait", "Input"],
  " to the scheduler, then it starts the ",
  StyleBox["ServerD", "Input"],
  " loop"
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`WaitD[task_] := 
      Module[\n\t\t{reply}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<wait\>", ToString[task]]; \n\t\t
        dist`serverDepth = dist`serverDepth + 1; \n\t\t
        reply = dist`ServerD[]; \n\t\t
        dist`serverDepth = dist`serverDepth - 1; \n\t\t
        ToExpression[reply[\([3]\)]]\n\t\t]\)], "Input",
  InitializationCell->True],

Cell["SelectD[taskList]", "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`SelectD[taskList_] := 
      Module[\n\t\t{len, \ reply, task, \ result, i}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        len = Length[taskList]; \n\t\t
        If[len == 0, \n\t\t\t
          \(dist`ErrorD["\<Task list must not be empty\>"]; \)\n\t\t\t]; \n
        \t\tdist`SendListD["\<nwait\>", taskList]; \n\t\t
        dist`serverDepth = dist`serverDepth + 1; \n\t\t
        reply = dist`ServerD[]; \n\t\t
        dist`serverDepth = dist`serverDepth - 1; \n\t\t
        task = ToExpression[reply[\([2]\)]]; \n\t\t
        result = ToExpression[reply[\([3]\)]]; \n\t\t
        For[i = 1, i <= len, \(i++\), \n\t\t\t
          \(If[taskList[\([i]\)] == task, \n\t\t\t\t\(Return[{i, result}]; 
            \)\n\t\t\t\t]; \)\n\t\t\t]; \n\t\t
        dist`ErrorD["\<Invalid reply\>"]; \n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["DeleteD[task]", "Input"],
  " deletes ",
  StyleBox["task", "Input"],
  " from the current session. If task is accessed again this will cause an \
error that will end the session."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`DeleteD[task_] := 
      Module[\n\t\t{}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<delete\>", ToString[task]]; \n\t\t"\<okay\>"\n\t]\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["FlushD[key]", "Input"],
  " flushes the internal result caches of the system. If ",
  StyleBox["key", "Input"],
  " is ",
  StyleBox["0", "Input"],
  ", then the result caching is switched off. If the key is ",
  StyleBox["1", "Input"],
  " then it remains switched on."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`FlushD[key_] := 
      Module[\n\t\t{}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        If[key != 0 && key != 1, \n\t\t\t
          \(dist`ErrorD["\<Invalid flush key (must be 0 or 1)\>"]; \)\n\t\t]; 
        \n\t\tdist`SendD["\<flush\>", ToString[key]]; \n\t\t"\<okay\>"\n\t]
        \)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ResetD[]", "Input"],
  " resets the session, i.e., it deletes all tasks and their results, flushes \
all caches, it destroys and restarts all ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " kernels connected to the system (except for the kernel connected to the \
user interface)."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ResetD[] := 
      Module[\n\t\t{reply}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<reset\>"]; \n\t\treply = dist`ReceiveD[]; \n\t\t
        If[reply[\([1]\)] != "\<okay\>", \n\t\t\t
          \(dist`ErrorD["\<Unexpected message (\>"<>reply[\([1]\)]<>"\<)\>"]; 
          \)\n\t\t\t]; \n\t\t"\<okay\>"\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["AbortD[message]", "Input"],
  " aborts the session, close all the remote ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " kernels and the corresponding connections and prints ",
  StyleBox["message", "Input"],
  " at the user interface."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`AbortD[message_] := 
      Module[\n\t\t{}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<abort\>", message]; \n\t\t
        If[dist`isClient != True, \n\t\t\tdist`initialized = False; \n\t\t\t
          Return[message]; \n\t\t]; \n\t\t"\<okay\>"\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["TraceD[filename]", "Input"],
  " creates the file ",
  StyleBox["filename", "Input"],
  ", that is used after the session is closed to create diagrams with the \
events of the session.\n",
  StyleBox["TraceD[]", "Input"],
  " clears the value of ",
  StyleBox["tracefile", "Input"],
  ", so that for the next session no such file is generated."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`TraceD[arguments___] := 
      Module[\n\t\t{}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        If[Length[{arguments}] == 0, \n\t\t\t\(dist`tracefile =. ; \), \n
          \t\t\t\(If[Length[{arguments}] == 1, \n\t\t\t\t
            dist`tracefile = {arguments}[\([1]\)], \n\t\t\t\t
            \(Return["\<Only one argument is allowed\>"]; \)\n\t\t\t]; \)\n
          \t\t]; \n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["LabelD[labelname]", "Input"],
  " creates the label ",
  StyleBox["labelname", "Input"],
  ", that may be used in the ",
  StyleBox["tracefile", "Input"],
  ". Each call of ",
  StyleBox["LabelD", "Input"],
  " has to use a different ",
  StyleBox["labelname", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`LabelD[string_] := 
      Module[\n\t\t{}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<label\>", string]; \n\t\t"\<okay\>"\n\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["VisualizeD[width,height,updates,length]", "Input"],
  " creates a visualization window of size ",
  StyleBox["width", "Input"],
  StyleBox[" x ",
    FontSlant->"Italic"],
  StyleBox["height,", "Input"],
  " which shows the behaviour of the system for the last ",
  StyleBox["length", "Input"],
  " seconds and it is updated each ",
  StyleBox["updates", "Input"],
  " seconds."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`VisualizeD[width_, height_, updates_, length_] := 
      Module[\n\t\t{}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<visual\>", ToString[width], ToString[height], 
          ToString[updates*1000], ToString[length*1000]]; \n\t\t"\<okay\>"\n
        \t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["DataD[]", "Input"],
  " creates a shared data. Initially the data has no value. The data is \
accessed through a handle, which is a unique number, whose value is taken \
from the available values for tasks. The function returns the handle."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`DataD[] := 
      Module[\n\t\t{t}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        If[dist`taskId\  == \ dist`taskMax, \ 
          \(dist`ErrorD["\<Task identifier range overflow\>"]; \)\n\t\t]; \n
        \t\tt = dist`taskId; \n\t\tdist`SendD["\<data\>", ToString[t]]; \n\t\t
        dist`taskId = t + 1; \n\t\tt\n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["PutD[id, value]", "Input"],
  " assigns ",
  StyleBox["value", "Input"],
  " to the shared data ",
  StyleBox["id", "Input"],
  ". "
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`PutD[id_, value_] := 
      Module[\n\t\t{}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<put\>", ToString[id], ToString[value]]; \n\t\t
        "\<okay\>"\n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["GetD[id]", "Input"],
  " returns the value of the shared data ",
  StyleBox["id", "Input"],
  ". It works exactly like ",
  StyleBox["WaitD", "Input"],
  ", therefore it just calls it. If ",
  StyleBox["id", "Input"],
  " doesn't have yet a value, it waits until a value is assigned to it."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`GetD[id_] := dist`WaitD[id]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ClearD[id]", "Input"],
  " unassigns the value of the shared data ",
  StyleBox["id", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ClearD[id_] := 
      Module[\n\t\t{}, \n\t\t
        If[dist`initialize\  != \ True, \ 
          \(dist`ErrorD["\<The system is not connected\>"]; \)\n\t\t]; \n\t\t
        dist`SendD["\<clear\>", ToString[id]]; \n\t\t"\<okay\>"\t\t\n\t\t]
        \)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ClientD[]", "Input"],
  " is called only for the remote kernels, when ",
  StyleBox["isClient", "Input"],
  "  is ",
  StyleBox["True", "Input"],
  " (see the end of the file). It initializes the kernel, assignes the value \
",
  StyleBox["True", "Input"],
  " to the variable ",
  StyleBox["dist`intialized", "Input"],
  ", sets the file ",
  StyleBox["dist`cout", "Input"],
  " to the standard output (which is already connected to the remote Java \
scheduler), checks the version with the scheduler, waits for the ",
  StyleBox["taskid", "Input"],
  " message with the corresponding values, and calls ",
  StyleBox["ServerD", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ClientD[] := 
      Module[\n\t\t{reply}, \n\t\tdist`initialized = True; \n\t\t
        SetOptions[$Output, PageWidth \[Rule] Infinity]; \n\t\t
        dist`cout = $Output; \n\t\tdist`ReadlineD[]; \n\t\t
        dist`CheckVersionD[]; \n\t\treply = dist`ReceiveD[]; \n\t\t
        If[reply[\([1]\)] != "\<taskid\>", \n\t\t\t
          \(dist`ErrorD["\<Unexpected message (\>"<>reply[\([1]\)]<>"\<)\>"]; 
          \)\n\t\t\t]; \n\t\tdist`TaskidMessage[reply]; \n\t\tdist`ServerD[]; 
        \n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["ServerD[]", "Input"],
  " is a loop generated by the waiting of messages. This is called imediately \
after the initialization of a remote kernel, to enable the receipt of tasks \
and the loop it is executed until the process is stopped. It is called also \
by ",
  StyleBox["WaitD", "Input"],
  ", to block the process until it gets the result. In the latter case, ",
  StyleBox["serverDepth", "Input"],
  " should be greater than ",
  StyleBox["0", "Input"],
  ". In this case the loop is ended imediately after the result is received."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`ServerD[] := 
      Module[\n\t\t{return = False, message, key}, \n\t\t
        While[return != True, \n\t\t\tmessage = dist`ReceiveD[]; \n\t\t\t
          key = message[\([1]\)]; \n
          \t\t\t (*
            If[dist`isClient != True, \n\t\t\t\t\ \ \(Print[key]; \), \n
              \t\t\t\t\(Write[verif, key]; \), \n\t\t\t\t
              \(Print["\<isClient n-are valoare\>"]; \)\n\t\t\t]; *) \n\t\t\t
          Switch[key, \n\t\t\t\t\t"\<result\>", \n\t\t\t\t
            \(If[dist`serverDepth > 0, \n\t\t\t\t\t\(return = True; \), \n
              \t\t\t\t\t\(dist`ErrorD["\<Unexpected message (result)\>"]; \)\n
              \t\t\t\t\t]; \), \n\t\t\t\t\t"\<task\>", \n\t\t\t\t
            \(dist`TaskMessage[message]; \), \n\t\t\t\t\t"\<all\>", \n\t\t\t\t
            \(dist`AllMessage[message]; \), \n\t\t\t\t\t"\<quit\>", \n\t\t\t\t
            Write[verif, "\<Am primit mesajul quit\>"]; \n\t\t\t\t
            \(dist`QuitMessage[message]; \), \n\t\t\t\t\t"\<abort\>", \n
            \t\t\t\t\(dist`AbortMessage[message]; \), \n\t\t\t\t_, \n\t\t\t\t
            \(dist`ErrorD["\<Unexpected message(\>"<>key<>"\<)\>"]; \)\n
            \t\t\t\t]; \n\t\t\t]; \n\t\tIf[return, \n\t\t\tmessage]\n\t]\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["TaskMessage[message]", "Input"],
  " is called by a process that received a task. ",
  StyleBox["message", "Input"],
  " contains the message received, i.e. the key ",
  StyleBox["task", "Input"],
  ", the ",
  StyleBox["id",
    FontSlant->"Italic"],
  " of the task and the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " expression with the attribute ",
  StyleBox["Hold", "Input"],
  " and the arguments included in a list. First the list is removed, then the \
expression is evaluated and the result is send to the scheduler."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`TaskMessage[message_] := 
      Module[\n\t\t{id, exp, val}, \n\t\tid = message[\([2]\)]; \n\t\t
        exp = message[\([3]\)]; \n\t\tval = ToExpression[exp, InputForm]; \n
        \t\tval = ReleaseHold[Delete[val, {1, 1, 0}]]; \n\t\t
        dist`SendD["\<result\>", id, ToString[val, InputForm]]; \n\t\t]\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["AllMessage[message]", "Input"],
  " is called by a process that received the message ",
  StyleBox["all", "Input"],
  ". It executes the command received in the task."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`AllMessage[message_] := 
      Block[\n\t\t{command}, \n\t\tcommand = message[\([2]\)]; \n\t\t
        ToExpression[command]; \n\t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["QuitMessage[message]", "Input"],
  " is called by a process that received the message ",
  StyleBox["quit", "Input"],
  ". It stops the current session, sending the message to be displayed."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`QuitMessage[message_] := 
      Module[\n\t\t{code}, \n\t\tcode = message[\([2]\)]; \n\t\t
        dist`ErrorD["\<Quit message received (code \>"<>code<>"\<)\>"]; \n
        \t\t]\)], "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["AbortMessage[message]", "Input"],
  " is called by a process that received the message ",
  StyleBox["abort", "Input"],
  ". It stops the current session, sending the message to be displayed."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`AbortMessage[message_] := 
      Module[\n\t\t{code}, \n\t\tcode = message[\([2]\)]; \n\t\t
        dist`ErrorD["\<abort message received (\>"<>code<>"\<)\>"]; \n\t]\)], 
  "Input",
  InitializationCell->True],

Cell[TextData[{
  StyleBox["TaskidMessage[message]", "Input"],
  " processes the data received with the message ",
  StyleBox["taskid", "Input"],
  ". It assigns the limits for the ",
  StyleBox["taskId", "Input"],
  " received from the scheduler to the variables ",
  StyleBox["taskMin", "Input"],
  " and ",
  StyleBox["taskMax", "Input"],
  " and initializes ",
  StyleBox["taskId", "Input"],
  "."
}], "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(dist`TaskidMessage[message_] := 
      Module[\n\t\t{}, \n\t\t
        dist`taskMin = dist`MsgInteger[message[\([2]\)]]; \n\t\t
        dist`taskMax = dist`MsgInteger[message[\([3]\)]]; \n\t\t
        dist`taskId = dist`taskMin; \n\t\tdist`SendD["\<okay\>"]; 
        \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \n\t\t]\)], "Input",
  InitializationCell->True],

Cell["\<\
This is the command executed after the program is loaded. For the \
remote kernels, the client loop is started. The error and warning messages \
are first redirected to a temporar file. The local kernel does not do \
anything.\
\>", "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(\(If[dist`isClient, \n\t$Messages = OpenWrite["\</tmp/distmath.msg\>"]; 
      \n\tverif = OpenWrite["\</home/ipau/tmp/control.txt\>"]; \n\t
      Write[verif, "\<Lansez clientul\>"]; \n\tdist`ClientD[]; \n]; \)\)], 
  "Input",
  InitializationCell->True],

Cell["\<\
These are only attempts to preprocess a task on the local kernel.\
\
\>", "Text",
  Background->GrayLevel[0.666667]],

Cell[BoxData[
    \(FunctionName[x_] := x[\([1, 0]\)]; \n
    FunctionArgs[x_] := ReleaseHold[ReplacePart[x, List, {{1, 0}}]]\)], 
  "Input"],

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
},
FrontEndVersion->"X 3.0",
ScreenRectangle->{{0, 1024}, {0, 768}},
NotebookAutoSave->True,
AutoGeneratedPackage->Automatic,
RulerUnits->"Millimeters",
CellGrouping->Manual,
WindowSize->{517, 598},
WindowMargins->{{225, Automatic}, {5, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "1"->{
    Cell[8239, 201, 584, 11, 187, "Input",
      InitializationCell->True,
      CellTags->"1"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"1", 50527, 1328}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1709, 49, 648, 15, 123, "Input",
  InitializationCell->True],
Cell[2360, 66, 65, 1, 27, "Input"],
Cell[2428, 69, 86, 2, 27, "Input",
  InitializationCell->True],
Cell[2517, 73, 4590, 84, 1499, "Input",
  InitializationCell->True],
Cell[7110, 159, 255, 5, 91, "Input"],
Cell[7368, 166, 83, 2, 27, "Input",
  InitializationCell->True],
Cell[7454, 170, 219, 5, 66, "Text"],
Cell[7676, 177, 94, 2, 27, "Input",
  InitializationCell->True],
Cell[7773, 181, 133, 4, 48, "Text"],
Cell[7909, 187, 127, 3, 27, "Input",
  InitializationCell->True],
Cell[8039, 192, 197, 7, 48, "Text"],
Cell[8239, 201, 584, 11, 187, "Input",
  InitializationCell->True,
  CellTags->"1"],
Cell[8826, 214, 222, 5, 66, "Text"],
Cell[9051, 221, 913, 14, 427, "Input",
  InitializationCell->True],
Cell[9967, 237, 127, 4, 48, "Text"],
Cell[10097, 243, 178, 4, 43, "Input",
  InitializationCell->True],
Cell[10278, 249, 89, 1, 48, "Text"],
Cell[10370, 252, 87, 2, 27, "Input",
  InitializationCell->True],
Cell[10460, 256, 267, 9, 66, "Text"],
Cell[10730, 267, 118, 2, 59, "Input",
  InitializationCell->True],
Cell[10851, 271, 151, 4, 48, "Text"],
Cell[11005, 277, 91, 2, 27, "Input",
  InitializationCell->True],
Cell[11099, 281, 93, 2, 27, "Input",
  InitializationCell->True],
Cell[11195, 285, 1568, 40, 282, "Text"],
Cell[12766, 327, 109, 2, 27, "Input",
  InitializationCell->True],
Cell[12878, 331, 1612, 29, 555, "Input",
  InitializationCell->True],
Cell[14493, 362, 166, 4, 48, "Text"],
Cell[14662, 368, 761, 14, 283, "Input",
  InitializationCell->True],
Cell[15426, 384, 637, 19, 102, "Text"],
Cell[16066, 405, 113, 2, 27, "Input",
  InitializationCell->True],
Cell[16182, 409, 564, 11, 203, "Input",
  InitializationCell->True],
Cell[16749, 422, 253, 8, 66, "Text"],
Cell[17005, 432, 106, 2, 27, "Input",
  InitializationCell->True],
Cell[17114, 436, 546, 10, 187, "Input",
  InitializationCell->True],
Cell[17663, 448, 247, 7, 66, "Text"],
Cell[17913, 457, 415, 8, 155, "Input",
  InitializationCell->True],
Cell[18331, 467, 232, 6, 66, "Text"],
Cell[18566, 475, 535, 10, 299, "Input",
  InitializationCell->True],
Cell[19104, 487, 185, 5, 48, "Text"],
Cell[19292, 494, 555, 11, 235, "Input",
  InitializationCell->True],
Cell[19850, 507, 317, 8, 66, "Text"],
Cell[20170, 517, 467, 8, 171, "Input",
  InitializationCell->True],
Cell[20640, 527, 294, 9, 66, "Text"],
Cell[20937, 538, 411, 8, 155, "Input",
  InitializationCell->True],
Cell[21351, 548, 632, 13, 120, "Text"],
Cell[21986, 563, 799, 14, 315, "Input",
  InitializationCell->True],
Cell[22788, 579, 226, 5, 66, "Text"],
Cell[23017, 586, 146, 3, 75, "Input",
  InitializationCell->True],
Cell[23166, 591, 423, 11, 84, "Text"],
Cell[23592, 604, 297, 6, 107, "Input",
  InitializationCell->True],
Cell[23892, 612, 345, 11, 66, "Text"],
Cell[24240, 625, 390, 7, 139, "Input",
  InitializationCell->True],
Cell[24633, 634, 421, 14, 84, "Text"],
Cell[25057, 650, 919, 16, 251, "Input",
  InitializationCell->True],
Cell[25979, 668, 390, 16, 66, "Text"],
Cell[26372, 686, 371, 7, 155, "Input",
  InitializationCell->True],
Cell[26746, 695, 277, 7, 66, "Text"],
Cell[27026, 704, 545, 10, 235, "Input",
  InitializationCell->True],
Cell[27574, 716, 1317, 25, 282, "Text"],
Cell[28894, 743, 897, 17, 283, "Input",
  InitializationCell->True],
Cell[29794, 762, 435, 15, 84, "Text"],
Cell[30232, 779, 587, 11, 235, "Input",
  InitializationCell->True],
Cell[30822, 792, 126, 4, 48, "Text"],
Cell[30951, 798, 803, 14, 283, "Input",
  InitializationCell->True],
Cell[31757, 814, 595, 11, 235, "Input",
  InitializationCell->True],
Cell[32355, 827, 267, 8, 66, "Text"],
Cell[32625, 837, 488, 10, 187, "Input",
  InitializationCell->True],
Cell[33116, 849, 68, 1, 48, "Text"],
Cell[33187, 852, 950, 18, 363, "Input",
  InitializationCell->True],
Cell[34140, 872, 255, 7, 66, "Text"],
Cell[34398, 881, 310, 7, 139, "Input",
  InitializationCell->True],
Cell[34711, 890, 345, 10, 66, "Text"],
Cell[35059, 902, 439, 9, 187, "Input",
  InitializationCell->True],
Cell[35501, 913, 367, 9, 84, "Text"],
Cell[35871, 924, 479, 9, 203, "Input",
  InitializationCell->True],
Cell[36353, 935, 322, 9, 66, "Text"],
Cell[36678, 946, 421, 8, 203, "Input",
  InitializationCell->True],
Cell[37102, 956, 419, 11, 102, "Text"],
Cell[37524, 969, 534, 10, 219, "Input",
  InitializationCell->True],
Cell[38061, 981, 351, 12, 66, "Text"],
Cell[38415, 995, 299, 6, 139, "Input",
  InitializationCell->True],
Cell[38717, 1003, 453, 13, 84, "Text"],
Cell[39173, 1018, 425, 8, 155, "Input",
  InitializationCell->True],
Cell[39601, 1028, 315, 6, 84, "Text"],
Cell[39919, 1036, 476, 9, 219, "Input",
  InitializationCell->True],
Cell[40398, 1047, 207, 8, 48, "Text"],
Cell[40608, 1057, 333, 7, 139, "Input",
  InitializationCell->True],
Cell[40944, 1066, 365, 10, 66, "Text"],
Cell[41312, 1078, 93, 2, 27, "Input",
  InitializationCell->True],
Cell[41408, 1082, 176, 6, 48, "Text"],
Cell[41587, 1090, 316, 7, 139, "Input",
  InitializationCell->True],
Cell[41906, 1099, 719, 20, 120, "Text"],
Cell[42628, 1121, 563, 10, 235, "Input",
  InitializationCell->True],
Cell[43194, 1133, 613, 13, 120, "Text"],
Cell[43810, 1148, 1281, 21, 523, "Input",
  InitializationCell->True],
Cell[45094, 1171, 617, 17, 102, "Text"],
Cell[45714, 1190, 376, 7, 139, "Input",
  InitializationCell->True],
Cell[46093, 1199, 242, 6, 66, "Text"],
Cell[46338, 1207, 199, 4, 91, "Input",
  InitializationCell->True],
Cell[46540, 1213, 265, 6, 66, "Text"],
Cell[46808, 1221, 245, 5, 91, "Input",
  InitializationCell->True],
Cell[47056, 1228, 267, 6, 66, "Text"],
Cell[47326, 1236, 234, 5, 91, "Input",
  InitializationCell->True],
Cell[47563, 1243, 448, 14, 84, "Text"],
Cell[48014, 1259, 842, 13, 155, "Input",
  InitializationCell->True],
Cell[48859, 1274, 285, 6, 84, "Text"],
Cell[49147, 1282, 275, 5, 107, "Input",
  InitializationCell->True],
Cell[49425, 1289, 126, 4, 48, "Text"],
Cell[49554, 1295, 141, 3, 43, "Input"],
Cell[49698, 1300, 66, 2, 27, "Input",
  InitializationCell->True],
Cell[49767, 1304, 73, 2, 27, "Input",
  InitializationCell->True]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

